import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:vdocipher_flutter/vdocipher_flutter.dart';
import 'package:vdocipher_flutter_example/controllers/vdo_custom_controller.dart';
import 'package:vdocipher_flutter_example/samples.dart';

class VdoPlaybackView extends StatefulWidget {
  final bool controls;

  const VdoPlaybackView({Key? key, this.controls = true}) : super(key: key);

  @override
  VdoPlaybackViewState createState() => VdoPlaybackViewState();
}

class VdoPlaybackViewState extends State<VdoPlaybackView> {
  VdoPlayerController? _controller;
  final double aspectRatio = 16 / 9;
  bool _isInPictureInPictureMode = false;
  VdoPlayerValue? vdoPlayerValue;
  final ValueNotifier<bool> _isFullScreen = ValueNotifier(false);

  @override
  Widget build(BuildContext context) {
    String? mediaId = ModalRoute.of(context)?.settings.arguments as String?;
    EmbedInfo embedInfo = sample_2;
    if (mediaId != null && mediaId.isNotEmpty) {
      embedInfo = EmbedInfo.offline(mediaId: mediaId);
    }
    return Scaffold(
        body: SafeArea(
      child: Column(crossAxisAlignment: CrossAxisAlignment.stretch, children: [
        Flexible(
            child: Stack(
          alignment: Alignment.center,
          children: [
            SizedBox(
              width: MediaQuery.of(context).size.width,
              height: _isFullScreen.value
                  ? MediaQuery.of(context).size.height
                  : _getHeightForWidth(MediaQuery.of(context).size.width),
              child: VdoPlayer(
                embedInfo: embedInfo,
                aspectRatio: aspectRatio,
                onError: _onVdoError,
                onFullscreenChange: _onFullscreenChange,
                onPlayerCreated: _onPlayerCreated,
                controls: widget.controls,
                onPictureInPictureModeChanged: _onPictureInPictureModeChanged,
              ),
            ),
            if (_controller == null ||
                widget.controls ||
                _isInPictureInPictureMode)
              const SizedBox.shrink()
            else
              SizedBox(
                width: MediaQuery.of(context).size.width,
                height: _isFullScreen.value
                    ? MediaQuery.of(context).size.height
                    : _getHeightForWidth(MediaQuery.of(context).size.width),
                child: VdoCustomControllerView(
                  controller: _controller,
                  onError: _onVdoError,
                  onFullscreenChange: _onFullscreenChange,
                ),
              )
          ],
        )),
        ValueListenableBuilder(
            valueListenable: _isFullScreen,
            builder: (context, dynamic value, child) {
              return value ? const SizedBox.shrink() : _nonFullScreenContent();
            }),
      ]),
    ));
  }

  _onVdoError(VdoError vdoError) {
    if (kDebugMode) {
      print("Oops, the system encountered a problem: ${vdoError.message}");
    }
  }

  _onPlayerCreated(VdoPlayerController? controller) {
    setState(() {
      _controller = controller;
      _onEventChange(_controller);
    });
  }

  _onPictureInPictureModeChanged(bool isInPictureInPictureMode) {
    setState(() {
      _isInPictureInPictureMode = isInPictureInPictureMode;
    });
  }

  _onEventChange(VdoPlayerController? controller) {
    controller!.addListener(() {
      VdoPlayerValue value = controller.value;
      setState(() {
        vdoPlayerValue = value;
      });
      if (kDebugMode) {
        print("VdoControllerListner"
            "\nloading: ${value.isLoading} "
            "\nplaying: ${value.isPlaying} "
            "\nbuffering: ${value.isBuffering} "
            "\nended: ${value.isEnded}");
      }
      _printProperties(controller);
    });
  }

  _printProperties(VdoPlayerController? controller) async {
    int? totalPlayed =
        await controller?.getPlaybackProperty("totalPlayed") as int?;
    int? totalCovered =
        await controller?.getPlaybackProperty("totalCovered") as int?;
    if (kDebugMode) {
      print("VdoControllerListener"
          "\ntotalPlayed: $totalPlayed "
          "\ntotalCovered: $totalCovered ");
    }
  }

  _onFullscreenChange(isFullscreen) {
    setState(() {
      _isFullScreen.value = isFullscreen;
    });
  }

  _nonFullScreenContent() {
    return Padding(
      padding: const EdgeInsets.all(8.0),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.center,
        children: const [
          Text(
            'Sample Playback',
            style: TextStyle(fontSize: 20.0),
          ),
        ],
      ),
    );
  }

  double _getHeightForWidth(double width) {
    return width / aspectRatio;
  }
}
